#ifndef _AD_SKIPPER_43_H
#define _AD_SKIPPER_43_H

extern dword fw_fChangePlaybackPos;   //12.88
extern dword fw_get_channel_info;
extern type_state playmode_str[];

enum {
    ASPECT_4_3    = 2,
    ASPECT_16_9   = 3,
    ASPECT_2_21_1 = 4,
};
enum {
    AFD0 = 0x8,
    AFD1 = 0x9,
    AFD2 = 0xa,
    AFD3 = 0xb,
    AFD4 = 0xc,
    AFD5 = 0xd,
    AFD6 = 0xe,
    AFD7 = 0xf,
};
enum {
    STATE_IDLE = 0,
    STATE_INIT,
    STATE_OPEN,
    STATE_UPDATE,
    STATE_CLOSE,
};

#define BYTESPERSECTOR      512
#define MAX_CUTPOINTS     0x100
#define TS_BUF_LEN        0x200000      //2Mbyte - what is optimim size?
#define ILENGTH_THRESHOLD 0x6000        //0x6000
#define IFILTER_THRESHOLD 0x1000000      //@4seconds:  1second @4Mbs = 0x80000
#define GLENGTH_THRESHOLD 0x18000        //0x6000
#define GFILTER_THRESHOLD 0x1000000      //@4seconds:  1second @4Mbs = 0x80000

//-----------------------------------------------------------------------------
//index file format
/*
    section_type            big endian, bdb0_xxxx
    section_length
    ...
    ...section...
    ...

    section_type
    section_length
    ...
    ...section...
    ...


    section_types
    bdb0_0001 - info
    bdb0_0002 - aspect
    bdb0_0003 - iframes
    bdb0_0004 - gframes
*/

enum {
    CUT_SECTION_MASK   = 0xbdb00000,
    CUT_SECTION_INFO   = 0xbdb00001,
    CUT_SECTION_ASPECT = 0xbdb00002,
    CUT_SECTION_IFRAME = 0xbdb00003,
    CUT_SECTION_GFRAME = 0xbdb00004,
};

typedef struct{
    dword type;             //CUT_SECTION_INFO - bdb0_0001
    dword length;
    char filename[0x80];
    dword loaded;
    dword scanning;
    dword scanned;
    dword analysed;
} type_info;

typedef struct{
    dword from;
    dword to;
    ulong64 fpos64;
} type_position;

typedef struct{
    dword length;
    ulong64 fpos64;
} type_length;

typedef struct{
    dword type;             //CUT_SECTION_ASPECT - bdb0_0002
    dword length;
    dword count;
    type_position *cut;
} type_xaspect;

typedef struct{
    dword type;             //CUT_SECTION_IFRAME,CUT_SECTION_GFRAME - bdb0_0003,4
    dword length;
    dword min_length;
    ulong64 min_fpos64;
    dword count;
    type_length *ilength;
} type_iframe;

typedef struct{
    type_info   info;
    type_xaspect aspect;
    type_iframe iframe;
    type_iframe gframe;
} type_section;

extern type_section section_playing;


typedef struct {
    int state;
    dword stime;
    int cutfile_list_position;
    int scanrate;
    int playmode;
    TYPE_File *Ip;
    ulong64 flen64;
    ulong64 fpos64;
    byte* ts_buf;
    dword ts_len;
    int tread;
    int twrite;
    int fine;
    int video_pid;
    int aspect_ratio_information;
    int flags;
    int abort;
} type_scan;

extern type_scan pb_scan;

extern int last_tvRadio;
extern int last_chNum;

//----------------------------------------------------------------------------
extern int automatic_skip_enabled;

extern int two_seconds_enabled;
extern dword two_seconds;

extern int debug;

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
extern void TSRCommanderConfigDialog();
extern bool TSRCommanderExitTAP();
extern int is_tsr_commander_running(void);

//----------------------------------------------------------------------------
//
//main
//
extern type_menu main_menu;

extern int TAP_Main(void);
extern int close(void);
extern dword TAP_EventHandler(word event, dword param1, dword param2);
extern int main_menu_handler(type_menu *menu, dword param1);
extern int key_handler(dword param1);

extern void init();

//----------------------------------------------------------------------------
//playback.c
extern int on_playback_started();
extern int on_playback_stopped();
extern int on_timeshift_playback_started();
extern int on_timeshift_playback_stopped();
extern int on_recording_playback_started();
extern int on_recording_playback_stopped();
extern int on_channel_change();  // == timeshift killed
extern int on_playback_aspect_change();
extern int on_recording_but_not_playing();

extern void oprintf(const void *fmt, ...);
extern void osd_status(type_section *section, type_scan *scan);
extern void two_second_update();

extern int skip_to_next(int ratio, int back);

//----------------------------------------------------------------------------
//cutfile.c
extern int load_cutfile(char *cut_filename, type_section *section);
extern int save_cutfile(type_section *section, char *cut_filename);

extern int create_section(type_section *section);
extern void free_section(type_section *section);
extern void dump_section(type_section *section);

//----------------------------------------------------------------------------
//scan.c
extern int scan_ts_for_aspect(ulong64 foffset64, type_section *section, type_scan *scan);
extern int scan_init(type_scan *scan, type_section *section, char *cut_filename);
extern int scan_open(type_scan *scan, type_section *section, char *cut_filename);
extern int scan_update(type_scan *scan, type_section *section);
extern int scan_close(type_scan *scan, type_section *section, char *cut_filename);
extern int playback_update(type_scan *scan, type_section *section);

//-----------------------------------------------------------------------------

#endif
